/*
 * tvprogxml
 * Copyright (c) 2003       Erik Waling <erikw@acc.umu.se>
 * Distributed under the terms of the GNU GPL. See the COPYING file
 * for a copy of the GNU Genereal Public License.
 */

typedef struct channel {
    char *channel_name;
    char *channel_id;
    char placement;
    struct content *content;
    
    struct channel *next;
} channel;

typedef struct content {
    char *title;
    char *desc;
    char *showview;
    time_t start_time;
    time_t end_time;
    char watch;
    char done_alarm;

    struct content *next;
} content;
    
typedef struct watches {
    char *regexp;
    char type;     /* 1 = name, 2 = description */ 
    struct watches *next;
} watches;

void read_xmltv_data(const char *xmltvfile);
void parse_xmltv_channels(xmlDocPtr doc, xmlNodePtr cur);
void parse_xmltv_programs(xmlDocPtr doc, xmlNodePtr cur);
time_t parse_xmltv_time(const char *datestr);
struct channel *add_channel(struct channel *ch, const char *channelname, const char *channelid);

void debug_chan_print(struct channel *ch);
void debug_watch_print(struct watches *w);

struct channel *add_program(struct channel *ch, const char *channelid, const char *title, const char *desc, time_t start, time_t stop, const char *showview);
char is_watch(struct watches *w, char *str, char type);
void flag_watches(struct channel *ch);
struct watches *free_watch_list(struct watches *w);
void get_programs(struct channel *c);
void get_channels(struct channel *ch);
struct channel *sort_channels(struct channel *ch);
void free_old_programs(struct channel *ch);
void fix_stop_time(struct channel *ch);
struct channel *find_search(char *findstr, struct channel *ch);
int count_programs(struct channel *ch);


